unit ExStdCtrl;

{*****************************************************************************}
{*                                                                           *}
{* TECustomEdit - has the ability to complete an user entry like excel.      *}
{*                After a change of the contens it search a supplement list. *}
{*              If the user entry match an the first letters of an entry of  *}
{*              the list then this entry is copied to the user entry field.  *}
{*              The letters that are completed the entry are marked as       *}
{*              selected.                                                    *}
{*              If more than one entry in the supplement list match the user *}
{*              entry, the first entry is used.                              *}
{*              The supplement list is a sorted string list with no          *}
{*              duplicates.                                                  *}
{*              The user can force the supplement by pressing ALT right.     *}
{*              Additional the control can show the full text if it's length *}
{*              expand the width of the controll. This is controled by the   *}
{*              properity "ShowClippedTextFull". If the Mouse moves over the *}
{*              control a timer starts. After "DelayOfShowTextFull" ms the   *}
{*              timer stops.                                                 *}
{*              The idea of this code is from Alexandre GUILLIEN, email      *}
{*              AGuillien@csi.com.                                           *}
{*              There are also to new events: "OnMouseIn" and "OnMouseOut"   *}
{*                                                                           *}
{*  TECustomMaskEdit - same functinality like above. If a Editmask is set,   *}
{*                     UseAutoSupplement is set automatic to False.          *}
{*                                                                           *}
{*  TECustomComboBox - same functinality like above.                         *}
{*                                                                           *}
{*****************************************************************************}

interface

uses
  StdCtrls, Classes, Messages, Controls;

Type
  TeCustomEdit = Class(TCustomEdit)
                 private
                   FMouseIn,
                   FboCharAction,
                   FboThreadRuning,
                   FboInternalChange,
                   FboLastNotFound,
                   FUseAutoSupplement    : Boolean;
                   FShowFull             : Boolean;
                   FDelayOfShowTextFull  : Word;
                   FSupplementList       : TStringList;
                   FThread               : TThread;
                   FOnMouseIn,
                   FOnMouseOut           : TNotifyEvent;
                   Procedure SetSupplementList(Value: TStringList);
                   Procedure ThreatDone(Sender : TObject);
                   Procedure DoMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
                 protected
                   property MouseIn : Boolean read FMouseIn;
                   property UseAutoSupplement : Boolean read  FUseAutoSupplement
                                                        write FUseAutoSupplement
                                                        default True;
                   property SupplementList : TStringList read FSupplementList
                                                         write SetSupplementList;
                   property ShowClippedTextFull : Boolean read FShowFull
                                                          write FShowFull
                                                          default True;
                   property DelayOfShowTextFull : Word read FDelayOfShowTextFull
                                                       write FDelayOfShowTextFull
                                                       default 500;
                   Property OnMouseIn : TNotifyEvent read FOnMouseIn write FOnMouseIn;
                   Property OnMouseOut : TNotifyEvent read FOnMouseOut write FOnMouseOut;
                   Procedure DoComplete;
                   Procedure Change; override;
                   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
                   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
                   Procedure MouseMove(Shift : TShiftState; X, Y : Integer); override;
                 public
                   constructor Create(AOwner: TComponent); override;
                   destructor Destroy; override;
                   Procedure Complete;
                   Function Clipped : Boolean;
                 end;
   TeEdit = class(TeCustomEdit)
            published
              property AutoSelect;
              property AutoSize;
              property BorderStyle;
              property CharCase;
              property Color;
              property Ctl3D;
              property DelayOfShowTextFull;
              property DragCursor;
              property DragMode;
              property Enabled;
              property Font;
              property HideSelection;
              property ImeMode;
              property ImeName;
              property MaxLength;
              property MouseIn;
              property OEMConvert;
              property ParentColor;
              property ParentCtl3D;
              property ParentFont;
              property ParentShowHint;
              property PasswordChar;
              property PopupMenu;
              property ReadOnly;
              property ShowHint;
              property TabOrder;
              property TabStop;
              property Text;
              property UseAutoSupplement;
              property SupplementList;
              property ShowClippedTextFull;
              property Visible;
              property OnChange;
              property OnClick;
              property OnDblClick;
              property OnDragDrop;
              property OnDragOver;
              property OnEndDrag;
              property OnEnter;
              property OnExit;
              property OnKeyDown;
              property OnKeyPress;
              property OnKeyUp;
              property OnMouseDown;
              property OnMouseMove;
              property OnMouseUp;
              property OnStartDrag;
              property OnMouseIn;
              property OnMouseOut;
            end;
   TeCustomComboBox =
            Class(TCustomComboBox)
              private
                   FMouseIn,
                   FboCharAction,
                   FboThreadRuning,
                   FboInternalChange,
                   FboLastNotFound,
                   FUseAutoSupplement    : Boolean;
                   FShowFull             : Boolean;
                   FDelayOfShowTextFull  : Word;
                   FThread               : TThread;
                   FOnMouseIn,
                   FOnMouseOut           : TNotifyEvent;
                   Procedure ThreatDone(Sender : TObject);
                   Procedure DoMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
                   Procedure SetUseAutoSupplement(Value : Boolean);
                 protected
                   property MouseIn : Boolean read FMouseIn;
                   property UseAutoSupplement : Boolean read  FUseAutoSupplement
                                                        write SetUseAutoSupplement
                                                        default True;
                   property ShowClippedTextFull : Boolean read FShowFull
                                                          write FShowFull
                                                          default True;
                   property DelayOfShowTextFull : Word read FDelayOfShowTextFull
                                                       write FDelayOfShowTextFull
                                                       default 500;
                   Property OnMouseIn : TNotifyEvent read FOnMouseIn write FOnMouseIn;
                   Property OnMouseOut : TNotifyEvent read FOnMouseOut write FOnMouseOut;
                   Procedure DoComplete; dynamic;
                   Procedure Change; override;
                   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
                   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
                   Procedure MouseMove(Shift : TShiftState; X, Y : Integer); override;
                 public
                   constructor Create(AOwner: TComponent); override;
                   destructor Destroy; override;
                   Procedure Complete;
                   Function Clipped : Boolean;
             end;
  TeComboBox =
             class(TeCustomComboBox)
             published
               property Style; {Must be published before Items}
               property Color;
               property Ctl3D;
               property DragMode;
               property DragCursor;
               property DropDownCount;
               property Enabled;
               property Font;
               property ImeMode;
               property ImeName;
               property ItemHeight;
               property Items;
               property MaxLength;
               property ParentColor;
               property ParentCtl3D;
               property ParentFont;
               property ParentShowHint;
               property PopupMenu;
               property ShowHint;
               property Sorted;
               property TabOrder;
               property TabStop;
               property Text;
               property Visible;
               property OnChange;
               property OnClick;
               property OnDblClick;
               property OnDragDrop;
               property OnDragOver;
               property OnDrawItem;
               property OnDropDown;
               property OnEndDrag;
               property OnEnter;
               property OnExit;
               property OnKeyDown;
               property OnKeyPress;
               property OnKeyUp;
               property OnMeasureItem;
               property OnStartDrag;
               { new properties }
               property MouseIn;
               property UseAutoSupplement;
               property ShowClippedTextFull;
               property DelayOfShowTextFull;
               property OnMouseIn;
               property OnMouseOut;
             end;

Procedure Register;

implementation

uses
  Windows, WinProcs, SysUtils, extCtrls, Forms, Graphics;

Type
  TTCustomEdit = class(TThread)
                 private
                   { Private declarations }
                   iListCount,
                   iStart,
                   iEnd      : Integer;
                   List      : TStrings;
                   sListStr,
                   sStr      : String;
                 protected
                   procedure Execute; override;
                   procedure GetListCount;
                   procedure GetListString;
                 public
                   iPos      : Integer;
                   constructor Create(Const aList : TStrings;
                                      Const aStr  : String);
                 end;
  THintWin     = class(THintWindow)
                 private
                   procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
                 protected
                   iDeltaX,
                   iDeltaY,
                   iDeltaHeight,
                   iShowDelay     : Integer;
                   sText          : String;
                   FTimer         : TTimer;
                   FboHintVisible : Boolean;
                   procedure DoTimer(Sender: TObject);
                   procedure DisplayHint;
                   procedure Paint; override;
                 public
                   MainControl : TWinControl;
                   constructor Create(AOwner: TComponent); override;
                   destructor Destroy; override;
                   Procedure InitTimer;
                   Procedure Stop;
                   procedure Start( Sender         : TObject;
                                    ShowDelay,
                                    DeltaX, DeltaY,
                                    DeltaHeight    : Integer;
                                    Text           : String);
                 end;

Var
  HintWin          : THintWin;

{****************************************************************************}
{*                                                                          *}
{*   TeCustomEdit                                                           *}
{*                                                                          *}
{****************************************************************************}

constructor TeCustomEdit.Create(AOwner: TComponent);

Begin
inherited Create(AOwner);
FMouseIn           := False;
FUseAutoSupplement := True;
FboThreadRuning    := False;
FboLastNotFound    := False;
FSupplementList    := TStringList.Create;
With FSupplementList Do
  Begin
  Sorted     := True;
  Duplicates := dupIgnore;
  end;
FShowFull := True;
FDelayOfShowTextFull := 500;
end;

destructor TeCustomEdit.Destroy;

Begin
FSupplementList.Free;
inherited Destroy;
end;

Procedure TeCustomEdit.Change;

Begin
inherited Change;
If FUseAutoSupplement and
   not FboInternalChange and
   FboCharAction Then
  DoComplete;
end;

Procedure TeCustomEdit.DoComplete;

Begin
{ If last search was unsucessfull, it's not nessesary to search again }
If FboLastNotFound Then
  Exit;
{ If an old thread is runing, terminate it. We have a new search }
If FboThreadRuning Then
  Begin
  FboThreadRuning := False;
  FThread.Terminate;
  end;
If Text > '' Then
  Begin
  { call Threat to complete }
  FThread := TTCustomEdit.Create(FSupplementList, Text);
  FThread.OnTerminate := ThreatDone;
  FboThreadRuning := True;
  end;
end;

procedure TeCustomEdit.KeyDown(var Key: Word; Shift: TShiftState);

Begin
{ Hide hint window if exist }
If HintWin.MainControl = Self Then
  HintWin.Stop;
inherited KeyDown(Key, Shift);
FboCharAction := (Key > VK_Help) or (Key = VK_Space);
{ If the user make a other action then adding a character, last search result
  is unimported }
If not FboCharAction Then
  FboLastNotFound := False;
If (Key = VK_Right) and (Shift = [ssAlt]) Then
  DoComplete;
end;

Procedure TeCustomEdit.SetSupplementList(Value: TStringList);

Begin
FSupplementList.Assign(Value);
end;

Procedure TeCustomEdit.ThreatDone(Sender : TObject);

Var
  iPos, i          : Integer;

Begin
If not FboThreadRuning Then
  Exit;
{ save pos in supplement list in iPos }
iPos := TTCustomEdit(Sender).iPos;
{ nothing found -> nothing to do }
If iPos = -1 Then
  Begin
  FboLastNotFound := True;
  Exit;
  end;
i := Length(Text);
FboInternalChange := True;
{ Move text form supplement list to edit control }
Text := FSupplementList.Strings[iPos];
{ mark supplement part of text }
SelStart := i;
SelLength := Length(Text) - i;
FboInternalChange := False;
end;

Procedure TeCustomEdit.Complete;

Begin
DoComplete;
end;

Procedure TeCustomEdit.MouseMove(Shift : TShiftState; X, Y : Integer);

Var
  iX, iY, iH       : Integer;

Begin
If not FMouseIn Then
  Begin
  FMouseIn := True;
  If Assigned(FOnMouseIn) Then
    FOnMouseIn(Self);
  end;
If FShowFull and (Shift = []) and Clipped Then
  Begin
  iX := 0;
  iY := 0;
  iH := Height - 4;
  If (BorderStyle = bsNone) Then
    Begin
    iX := 2;
    iY := 1;
    iH := iH;
    end
  Else
    If Ctl3D Then
      Begin
      iX := 1;
      iY := 1;
      iH := iH - 3;
      end;
  { Start FTimer for cliped Text }
  HintWin.Start(Self, FDelayOfShowTextFull, iX, iY, iH, Text);
  end;
inherited MouseMove(Shift, X, Y);
end;

Function TeCustomEdit.Clipped : Boolean;

Var
  iModifier        : Integer;

Begin
HintWin.Canvas.Font.Assign(Font);
iModifier := 0;
If BorderStyle = bsSingle Then
  iModifier := 4;
Result := HintWin.Canvas.TextWidth(Text) > (Width - iModifier);
end;

Procedure TeCustomEdit.DoMouseLeave(var Msg: TMessage);

Begin
FMouseIn := False;
If HintWin.MainControl = Self Then
  Begin
  HintWin.Stop;
  HintWin.MainControl := nil;
  end;
If Assigned(FOnMouseOut) Then
  FOnMouseOut(Self);
inherited;
end;

Procedure TeCustomEdit.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

Begin
{ We don't want the hint window now }
If HintWin.MainControl = Self Then
  HintWin.Stop;
inherited;
end;

{****************************************************************************}
{*                                                                          *}
{*   TeCustomComboBox                                                       *}
{*                                                                          *}
{****************************************************************************}

constructor TeCustomComboBox.Create(AOwner: TComponent);

Begin
inherited Create(AOwner);
Sorted               := True;
FMouseIn             := False;
FUseAutoSupplement   := True;
FboThreadRuning      := False;
FboLastNotFound      := False;
FShowFull            := True;
FDelayOfShowTextFull := 500;
end;

destructor TeCustomComboBox.Destroy;

Begin
inherited Destroy;
end;

Procedure TeCustomComboBox.SetUseAutoSupplement(Value : Boolean);

Begin
If Value <> FUseAutoSupplement Then
  Begin
  FUseAutoSupplement := Value;
  If FUseAutoSupplement Then
    Sorted := True;
  end;
end;

Procedure TeCustomComboBox.Change;

Begin
inherited Change;
If FUseAutoSupplement and
   not FboInternalChange and
   FboCharAction Then
  DoComplete;
end;

Procedure TeCustomComboBox.DoComplete;

Begin
{ If last search was unsucessfull, it's not nessesary to search again }
If FboLastNotFound Then
  Exit;
{ If an old thread is runing, terminate it. We have a new search }
If FboThreadRuning Then
  Begin
  FboThreadRuning := False;
  FThread.Terminate;
  end;
If Text > '' Then
  Begin
  { call Threat for supplement }
  FThread := TTCustomEdit.Create(Items, Text);
  FThread.OnTerminate := ThreatDone;
  FboThreadRuning := True;
  end;
end;

procedure TeCustomComboBox.KeyDown(var Key: Word; Shift: TShiftState);

Begin
{ Hide hint window if exist }
If HintWin.MainControl = Self Then
  HintWin.Stop;
inherited KeyDown(Key, Shift);
FboCharAction := (Key > VK_Help) or (Key = VK_Space);
{ If the user make a other action then adding a character, last search result
  is unimported }
If not FboCharAction Then
  FboLastNotFound := False;
If (Key = VK_Right) and (Shift = [ssAlt]) Then
  DoComplete;
end;

Procedure TeCustomComboBox.ThreatDone(Sender : TObject);

Var
  iPos, i          : Integer;

Begin
If not FboThreadRuning Then
  Exit;
{ save pos in supplement list in iPos }
iPos := TTCustomEdit(Sender).iPos;
{ nothing found -> nothing to do }
If iPos = -1 Then
  Begin
  FboLastNotFound := True;
  Exit;
  end;
i := Length(Text);
FboInternalChange := True;
{ Move text form supplement list to edit control }
Text := Items[iPos];
{ mark supplement part of text }
SelStart := i;
SelLength := Length(Text) - i;
FboInternalChange := False;
end;

Procedure TeCustomComboBox.Complete;

Begin
DoComplete;
end;

Procedure TeCustomComboBox.MouseMove(Shift : TShiftState; X, Y : Integer);

Var
  iX, iY, iH       : Integer;

Begin
If not FMouseIn Then
  Begin
  FMouseIn := True;
  If Assigned(FOnMouseIn) Then
    FOnMouseIn(Self);
  end;
If FShowFull and (Shift = []) and Clipped Then
  Begin
  iX := -1;
  iY := -1;
  iH := Height - 5;
  { Start FTimer for cliped Text }
  HintWin.Start(Self, FDelayOfShowTextFull, iX, iY, iH, Text);
  end;
inherited MouseMove(Shift, X, Y);
end;

Function TeCustomComboBox.Clipped : Boolean;

Var
  iModifier        : Integer;

Begin
HintWin.Canvas.Font.Assign(Font);
{iModifier := 0;
If BorderStyle = bsSingle Then}
  iModifier := 4;
Result := HintWin.Canvas.TextWidth(Text) > (Width - iModifier);
end;

Procedure TeCustomComboBox.DoMouseLeave(var Msg: TMessage);

Begin
FMouseIn := False;
If HintWin.MainControl = Self Then
  Begin
  HintWin.Stop;
  HintWin.MainControl := nil;
  end;
If Assigned(FOnMouseOut) Then
  FOnMouseOut(Self);
inherited;
end;

Procedure TeCustomComboBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

Begin
{ We don't want the hint window now }
If HintWin.MainControl = Self Then
  HintWin.Stop;
inherited;
end;

{****************************************************************************}
{*                                                                          *}
{*   Threat                                                                 *}
{*                                                                          *}
{****************************************************************************}
{
  Der Threat sucht die Liste der Referenzeintrge durch. Die Liste muss sortiert
  sein. Verglichen werden nur die ersten Buchstaben. Diese mssen mit aStr
  bereinstimmen. Gibt es mehrere Eintrge die im Anfang bereinstimmen, so
  wird der ausgewhlt, der am weitesten forne in der Liste steht.
  Die Variable iPos enthlt das Suchergebnis. Ist es -1, so wurde keine
  bereinstimmung gefunden. Ansonsten ist es die Position in der Stringliste.
}
{ TTCustomEdit }

constructor TTCustomEdit.Create(Const aList : TStrings; Const aStr : String);

Begin
List := aList;
{ allways compare case insensetiv }
sStr := AnsiUpperCase(aStr);
FreeOnTerminate := True;
inherited Create(False);
end;

procedure TTCustomEdit.Execute;

Var
  s                : String;

begin
try
  { Place thread code here }
  Synchronize(GetListCount);
  { ensure that there is something to compare }
  If iListCount = 0 Then
    Begin
    iPos := -1;
    Exit;
    end;
  { I use a simple search algorithm }
  iStart := 0;
  iEnd := iListCount - 1;
  While (iStart < iEnd) and not Terminated Do
    Begin
    { determine mid of range }
    iPos := iStart + (iEnd - iStart) div 2;
    Synchronize(GetListString);  { sListStr := List.Strings[iPos]}
    s := AnsiUpperCase(Copy(sListStr, 1, Length(sStr)));
    If sStr >= s Then
      iStart := iPos;
    If sStr <= s Then
      iEnd := iPos;
    If iEnd = iStart + 1 Then
      If sStr < s Then
        iEnd := iStart
      Else
        iStart := iEnd;
    end; { While }
  Synchronize(GetListString);  { sListStr := List.Strings[iPos]}
  s := AnsiUpperCase(Copy(sListStr, 1, Length(sStr)));
  If (sStr = s) and not Terminated Then
    Begin
    { Goto first entry beginning with letters in s }
    Repeat
      Dec(iPos);
      If iPos >= 0 Then
        Synchronize(GetListString);  { sListStr := List.Strings[iPos] }
    Until Terminated or
          (iPos < 0) or
          (AnsiUpperCase(Copy(sListStr, 1, Length(sStr))) <> s);
    Inc(iPos);
    end
  Else
    iPos := -1;
except
  iPos := -1;
end; { try }
end;

Procedure TTCustomEdit.GetListCount;
{ Copy List.Count to iListCount }

Begin
iListCount := List.Count;
end;

Procedure TTCustomEdit.GetListString;
{ Copy List.strings[iPos] to sliststr }

Begin
sListStr := List.Strings[iPos];
end;

{****************************************************************************}
{*                                                                          *}
{*   THintWin                                                               *}
{*                                                                          *}
{****************************************************************************}
{
  Vom HintWin gibt es nur eine Instanz. ber die Variable HintWin kann darauf
  zugegriffen werden. HintWin muss nur einmal ber die Procedure Start
  angestossen werden. Es wartet eine vorgegebene Zeit uns zeigt dann den Text
  wie eine Fly-over Hilfe an.
  Mit der Funktion Stop wird das Fenster geschlossen bzw. der Timer angehalten.

  Procedure Start( Sender         : TWinControl;
                   ShowDelay,
                   DeltaX, DeltaY,
                   DeltaHeight    : Integer;
                   Text           : String);
  Sender ist das Objekt mit dem zu kleinen Anzeigebereich, beim Aufruf also Self.
  ShowDeley ist der Zeit in ms die vergeht, bis der Text in der Variablen Text
  angezeigt wird.
  HintWin wird normalerweise ein Fenster anzeigen, dass der Grsse von Sender
  entspricht. Eigenschaften wie Borderstyle und Ctl3D knnen abweischungen
  erfordern. Diese werden in den drei Werten DeltaX, DeltaY und DeltaHeigt
  angegeben. Die Position des Fensters wird um die Delta-Werte verschoben.
  DeltaHeight gibt abweichend die Hhe in Pixeln an.
  Achtung: HintWin kontrolliert NICHT ob ein Text wirklich zu gross ist. Wird
           HintWin ohne entsprechende berprfung aufgerufen, erscheint das
           Anzeigefenster innerhalb des Aufrufenden Controlls.
}

constructor THintWin.Create(AOwner: TComponent);

Begin
inherited Create(AOwner);
Color:= clInfoBk;
FTimer:= TTimer.Create(Self);
FTimer.Enabled := False;
FTimer.OnTimer:= DoTimer;
end;

destructor THintWin.Destroy;
begin
FTimer.Free;
inherited Destroy;
end;

Procedure THintWin.WMNCPaint(Var Message : TMessage);
{ Note : these repaint functions allow the window to be more beautiful
         a full black rectangle instead of an half white/half black one }

Begin
Canvas.Handle := GetWindowDC(Handle);
With Canvas Do
  try
    WinProcs.Rectangle(Canvas.Handle, 0, 0, Width, Height);
  finally
    Canvas.Handle := 0;
  end;
end;

Procedure THintWin.Paint;

Var
  R                : TRect;
  C                : PChar;

Begin
R := ClientRect;
Inc(R.Left, 1);
Inc(R.Top, 1);
Dec(R.Right, 1);
Dec(R.Bottom, 1);
Canvas.Font.Color := clInfoText;
GetMem(C, Length(Caption) + 1);
DrawText(Canvas.Handle, StrPCopy(C, Caption), -1, R, DT_LEFT or DT_NOPREFIX or
         DT_WORDBREAK);
FreeMem(C, Length(Caption) + 1);
end;

Procedure THintWin.Start( Sender          : TObject;
                          ShowDelay,
                          DeltaX, DeltaY,
                          DeltaHeight     : Integer;
                          Text            : String);

Begin
If MainControl = nil Then
  Begin
  iShowDelay := ShowDelay;
  iDeltaX := DeltaX;
  iDeltaY := DeltaY;
  iDeltaHeight := DeltaHeight;
  sText := Text;
  MainControl := TeCustomEdit(Sender);
  InitTimer;
  end;
end;

Procedure THintWin.InitTimer;

Begin
FTimer.Enabled:= False;
If (MainControl <> nil) Then
  Begin
  FTimer.Interval:= iShowDelay;
  FTimer.Enabled:= True;
  end;
end;

Procedure THintWin.DoTimer(Sender: TObject);

Begin
{ Turn off timer }
FTimer.Enabled := False;
DisplayHint;
end;

Procedure THintWin.DisplayHint;

Var
  Rect             : TRect;

Begin
{ Did we have an valid control? }
If MainControl = nil Then
  Exit;
FboHintVisible := True;
{ calc pos of window }
With Rect Do
  Begin
  Left        := 0;
  Top         := 0;
  TopLeft     := MainControl.ClientToScreen(Rect.TopLeft);
  Right       := HintWin.Canvas.TextWidth(sText) + 6;
  Bottom      := iDeltaHeight;
  BottomRight := MainControl.ClientToScreen(Rect.BottomRight);
  Dec(Top, iDeltaY);
  Dec(Left, iDeltaX);
  end; { With Rect }
HintWin.ActivateHint(Rect, sText);
end;

Procedure THintWin.Stop;

Begin
FTimer.Enabled := False;
If FboHintVisible Then
  Begin
  FboHintVisible := False;
  HintWin.ReleaseHandle;
  end;
end;

Procedure Register;

Begin
RegisterComponents('Standard', [TeEdit, TeComboBox]);
end;

initialization

Begin
HintWin := THintWin.Create(nil);
end;

end.
